"""Various tools to emerge and to show status for conky."""
__version__ = '0.3.5'

portage_tmpdir = '/var/tmp/portage'  # nosec
tmplogfile = '/var/log/portage/tmerge/tmp.emerge.log'
tmerge_logfile = '/var/log/portage/tmerge/emerge.log'
emerge_logfile = '/var/log/emerge.log'
server = '89.16.167.134'
dev_null = '/dev/null 2>&1'
glsa_webpage = 'https://security.gentoo.org/glsa/feed.rss1'
